[
    {
        author  => "Mario Zieschang",
        id      => "001.01-maz",
        entries => [
            {
                action   => "table.add",
                name     => "client",
                template => "tpl_std"
            },
            {
                action   => "table.add",
                name     => "company",
                template => "tpl_std_client_ref"
            }
        ]
    }
]
