use strict;
use warnings;
use Test::More tests => 11;

require_ok('Geoffrey');
use_ok 'Geoffrey';

require_ok('DBI');
use_ok 'DBI';

require_ok('Hash::MD5');
use_ok 'Hash::MD5';

require_ok('DBI');
use_ok 'DBI';

require_ok('Getopt::Long');
use_ok 'Getopt::Long';

use_ok 'Geoffrey';
